/*
 * aTunes 1.13.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.controls;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.PopupMenu;
import java.awt.TrayIcon;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JDialog;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

/*
 * This class is based on Java6Tray class from TVBrowser project
 * and JTrayIcon by Alexander Potochkin
 * 
 * http://sourceforge.net/projects/tvbrowser/
 * http://weblogs.java.net/blog/alexfromsun/archive/2008/02/jtrayicon_updat.html
 */
/**
 * Tray icon with a Swing popup menu.
 */
public class JTrayIcon extends TrayIcon {

    /**
     * This special JPopupMenu prevents the user from removing the popup menu
     * listener.
     */
    public class JTrayIconPopupMenu extends JPopupMenu {

        private static final long serialVersionUID = -220434547680783992L;

        @Override
        public void removePopupMenuListener(PopupMenuListener l) {
            if (l == popupMenuListener) {
                return;
            }
            super.removePopupMenuListener(l);
        }
    }

    JDialog trayParent;
    Window trayParentParent;
    private JPopupMenu popupMenu;
    private MouseListener trayIconMouseListener;
    PopupMenuListener popupMenuListener;

    /**
     * Instantiates a new tray icon.
     * 
     * @param image
     *            the image
     */
    public JTrayIcon(Image image) {
        this(image, null);
    }

    /**
     * Instantiates a new tray icon.
     * 
     * @param image
     *            the image
     * @param trayParentParent
     *            the parent of the tray parent
     */
    public JTrayIcon(Image image, Window trayParentParent) {
        super(image);
        this.trayParentParent = trayParentParent;
    }

    @Override
    public synchronized void removeMouseListener(MouseListener listener) {
        if (listener == trayIconMouseListener) {
            return;
        }
        super.removeMouseListener(listener);
    }

    /**
     * Sets the popup menu.
     * 
     * @param popup
     *            the new popup menu
     */
    public void setJTrayIconJPopupMenu(JTrayIconPopupMenu popup) {
        popupMenu = popup;
        popupMenuListener = new PopupMenuListener() {
            @Override
            public void popupMenuCanceled(PopupMenuEvent e) {
                if (trayParent != null) {
                    trayParent.dispose();
                    trayParent = null;
                }
            }

            @Override
            public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
                if (trayParent != null) {
                    trayParent.dispose();
                    trayParent = null;
                }
            }

            @Override
            public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
                // Nothing to do
            }
        };
        popupMenu.addPopupMenuListener(popupMenuListener);
        setTrayIconMouseListener();
    }

    @Override
    public void setPopupMenu(PopupMenu popup) {
        throw new UnsupportedOperationException("use setJTrayIconPopupMenu(JTrayIconPopupMenu popup) instead");
    }

    /**
     * Sets the tray icon mouse listener.
     */
    private void setTrayIconMouseListener() {
        if (trayIconMouseListener == null) {
            trayIconMouseListener = new MouseAdapter() {
                @Override
                public void mousePressed(MouseEvent e) {
                    showPopup(e);
                }

                @Override
                public void mouseReleased(MouseEvent e) {
                    showPopup(e);
                }
            };
            addMouseListener(trayIconMouseListener);
        }
    }

    /**
     * Compute the proper position for a popup.
     * 
     * @param x
     *            the x
     * @param y
     *            the y
     * @param dim
     *            the dim
     * 
     * @return the point
     */
    private Point computeDisplayPoint(int x, int y, Dimension dim) {
        int computedX = x;
        int computedY = y;
        if (computedX - dim.width > 0) {
            computedX -= dim.width;
        }
        if (computedY - dim.height > 0) {
            computedY -= dim.height;
        }
        return new Point(computedX, computedY);
    }

    /**
     * Shows the popup menu.
     * 
     * @param e
     *            the mouse event
     */
    void showPopup(final MouseEvent e) {
        if (e.isPopupTrigger() && popupMenu != null) {
            if (trayParent == null) {
                trayParent = new JDialog(trayParentParent);
                trayParent.setUndecorated(true);
                trayParent.setAlwaysOnTop(true);
                Point displayPoint = computeDisplayPoint(e.getX(), e.getY(), popupMenu.getPreferredSize());
                trayParent.setVisible(true);
                popupMenu.show(trayParent, displayPoint.x - trayParent.getLocation().x, displayPoint.y - trayParent.getLocation().y);
                // popup works only for focused windows
                trayParent.toFront();
            }
        }
    }

}
